import "../root";
import "./slickgrid";
import "widgets/tables.css";
export declare const bk_data_table = "bk-data-table";
export declare const bk_cell_index = "bk-cell-index";
export declare const bk_header_index = "bk-header-index";
export declare const bk_cell_editor = "bk-cell-editor";
export declare const bk_cell_select = "bk-cell-select";
